/*
 *  MetaInterpolation.cpp
 *  treemusic
 *
 *  Created by Michael Chinen on 2/7/09.
 *  Copyright 2009 Michael Chinen. All rights reserved.
 *
 */

#include "MetaInterpolation.h"
#include "ParameterList.h"
#include "handyfuncs.h"
#include "TreeNode.h"


MetaInterpolation::MetaInterpolation(Interpolation* target,int bottomLayer, int topLayer,int numTopLayers,TreeNode* owner)
      :Interpolation(bottomLayer, topLayer, numTopLayers, owner)
{
   mTarget=target;
}

void MetaInterpolation::Randomize(double startTime, double duration,std::vector<TreeNode*> *activeNodes, std::vector<ParameterList*> *parameters, int layerNum,  int topLayer, int numControlledLayers, int param)
{

   mType=(InterpolationType)randint(eNumInterpolationTypes);
   mDoesConcatenate = true;//randint(2);

   switch(mType)
	{
		case ePeriodic:
      case eSquare:
			interpTypeVariable = (int)randfloatexp2(mymax(5,duration*3)); //can't have more periods then we have samples.
         mInterpParams.SetParameter(eInterpTypeVariable, (int)randfloatexp2(mymax(5,duration*3)),0.0,mymax(5,duration*3)+0.99);
			break;
         
	  case eExp2Step:
      case eStep:
         interpTypeVariable = randint(mymax(1.0,10*duration))+1; //can't have more periods then we have samples.
         mInterpParams.SetParameter(eInterpTypeVariable,interpTypeVariable,interpTypeVariable,interpTypeVariable);
			break;
	}
   mStartTime = startTime;
	mDuration = duration;
   mParam = (Parameter) 0;//only one for now (interpTypeParam)
   
    double startValue = mTarget->GetInterpParameters()->GetValue(mParam);
   //- we don't know where it will be at the end of the leaf.  
   //now check to see where the parameter will be at upon the end of the leaf.  
   //we do this by asking each of the active nodes what their value will be at the end of THIS leaf
   //and multiply them against the start value.
   
   double topValue = startValue;
   double bottomValue = startValue;

   for(int i =topLayer;i<numControlledLayers+1;i++)
   {
   //double GetLayerCoef(int layer, int paramIndex);
      topValue = mymax(topValue,startValue*mTarget->GetInterpParameters()->GetLayerCoef(i<numControlledLayers?i:layerNum,mParam));
      bottomValue = mymin(bottomValue, startValue*mTarget->GetInterpParameters()->GetLayerCoef(i<numControlledLayers?i:layerNum,mParam));
   }
   


   for(int j=0;j<mTarget->GetNumMetaInterpolations();j++)
   {
         
      if(mTarget->GetMetaInterpolation(j)->GetParameter() == mParam)
      {
         //we use this node's start time, and the parent node's end time (since we can't push them to overstep their boundries.)
         topValue = mymax(topValue,topValue*mTarget->GetMetaInterpolation(j)->MaxValueInTimeRange(mStartTime,mTarget->GetMetaInterpolation(j)->GetStartTime()+mTarget->GetMetaInterpolation(j)->GetDuration()));
         bottomValue = mymin(bottomValue,bottomValue*mTarget->GetMetaInterpolation(j)->MinValueInTimeRange(mStartTime,mTarget->GetMetaInterpolation(j)->GetStartTime()+mTarget->GetMetaInterpolation(j)->GetDuration()));
      }
   }
   
   
   assert(topValue >= bottomValue);
   //now decide if we are going to interpolate UP or down.
   bool up = coinflip();
    
   if(up)
   {
      if(mParam!=eBandwidth)
//         mEndCoef = 1.0+randfloatexp2( mymax(0.0,parameters->GetMaxValue(mParam)/mymax(0.000001,mymax(startValue,endValue)) -1.0));
         mEndCoef = 1.0+randfloatexp2( mymax(0.0,mTarget->GetInterpParameters()->GetMaxValue(mParam)/mymax(0.000001,topValue) -1.0));
      else
//               mEndCoef = 1.0+randfloat( mymax(0.0,parameters->GetMaxValue(mParam)/mymax(0.000001,mymax(startValue,endValue)) -1.0));
         mEndCoef = 1.0+randfloat( mymax(0.0,mTarget->GetInterpParameters()->GetMaxValue(mParam)/mymax(0.000001,topValue) -1.0));

   assert(topValue*mEndCoef<=mTarget->GetInterpParameters()->GetMaxValue(mParam)*1.02);
   }
   else
   {
	  //use the minimum to find out where the coefficient should be.  
	  //mEnd Coef can be from the ratio of min to the end, up to 1.0 
     
      if(mParam!=eBandwidth)
         mEndCoef = 1.0- randfloatexp2( 1.0-mymin(1.0,mymax(0.0,mTarget->GetInterpParameters()->GetMinValue(mParam)/mymax(0.000001,bottomValue))));
//         mEndCoef = mymax(0.0,parameters->GetMinValue(mParam)/mymax(0.000001,bottomValue))+ randfloatexp2( 1.0-mymax(0.0,parameters->GetMinValue(mParam)/mymax(0.000001,bottomValue)));   
     else
         mEndCoef = 1.0- randfloat( 1.0-mymin(1.0,mymax(0.0,mTarget->GetInterpParameters()->GetMinValue(mParam)/mymax(0.000001,bottomValue))));
//         mEndCoef = mymax(0.0,parameters->GetMinValue(mParam)/mymax(0.000001,bottomValue))+ randfloat( 1.0-mymax(0.0,parameters->GetMinValue(mParam)/mymax(0.000001,bottomValue)));   
     
   assert(bottomValue*mEndCoef>=mTarget->GetInterpParameters()->GetMinValue(mParam)*0.98);
   }
   //fix boundries.
   //if(topValue*mEndCoef>=parameters->GetMaxValue(mParam))
   //   mEndCoef = parameters->GetMaxValue(mParam)/topValue;
  // else if(bottomValue*mEndCoef<=parameters->GetMinValue(mParam))
    //  mEndCoef=parameters->GetMinValue(mParam)/bottomValue;
   
   //we might have caused a conflict on the other side, if we did, this guy can't interpolate, and must stay at 1.0
   //it can have local non concatenating fluctuations, however.
   if((topValue*mEndCoef>=mTarget->GetInterpParameters()->GetMaxValue(mParam)) ||
         (bottomValue*mEndCoef<=mTarget->GetInterpParameters()->GetMinValue(mParam)) )
         mEndCoef=1.0;
         

}

void MetaInterpolation::Apply(double time, std::vector<double>* params)
{
   //we simply concat our value with the param list, since Randomize has already checked boundries.
   (*params)[mParam]*=ValueAtTime(time);
}

void MetaInterpolation::Attach()
{
   mTarget->AddMetaInterpolation(this);
}

void MetaInterpolation::Detach()
{
   mTarget->ConcatAndRemoveMetaInterpolation(this);
}
